@@ -17,8 +17,8 @@ Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this program; if not, write to the
-Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+Boston, MA 02110-1301, USA.
 
 Derived from eel-background.c and eel-gdk-pixbuf-extensions.c by
 Darin Adler <darin@eazel.com> and Ramiro Estrugo <ramiro@eazel.com>
@@ -84,6 +84,7 @@ struct _FileSize
  * bad dithering when tiling the gradient
  */
 #define GRADIENT_PIXMAP_TILE_SIZE 128
+#define THUMBNAIL_SIZE 256
 
 typedef struct FileCacheEntry FileCacheEntry;
 #define CACHE_SIZE 4
@@ -298,6 +299,12 @@ bg_gsettings_mapping (GVariant *value,
 	const gchar *bg_key_value;
 	char *filename = NULL;
 
+	/* The final fallback if nothing matches is with a NULL value. */
+	if (value == NULL) {
+		*result = NULL;
+		return TRUE;
+	}
+
 	bg_key_value = g_variant_get_string (value, NULL);
 
 	if (bg_key_value && *bg_key_value != '\0') {
@@ -1080,6 +1087,8 @@ gnome_bg_get_pixmap_size (GnomeBG   *bg,
  * TRUE, the surface created will be created by a temporary X server connection
  * so that if someone calls XKillClient on it, it won't affect the application
  * who created it.
+ *
+ * Returns: %NULL on error (e.g. out of X connections)
  **/
 cairo_surface_t *
 gnome_bg_create_surface (GnomeBG	    *bg,
@@ -1115,7 +1124,10 @@ gnome_bg_create_surface (GnomeBG	    *bg,
                                                              CAIRO_CONTENT_COLOR,
                                                              pm_width, pm_height);
 	}
-	
+
+	if (surface == NULL)
+		return NULL;
+
 	cr = cairo_create (surface);
 	if (!bg->filename && bg->color_type == G_DESKTOP_BACKGROUND_SHADING_SOLID) {
 		gdk_cairo_set_source_color (cr, &(bg->primary));
@@ -1171,12 +1183,13 @@ gnome_bg_is_dark (GnomeBG *bg,
 	pixbuf = get_pixbuf_for_size (bg, -1, width, height);
 	if (pixbuf) {
 		GdkRGBA argb;
+		guchar a, r, g, b;
 
 		pixbuf_average_value (pixbuf, &argb);
-		guchar a = argb.alpha * 0xff;
-		guchar r = argb.red * 0xff;
-		guchar g = argb.green * 0xff;
-		guchar b = argb.blue * 0xff;
+		a = argb.alpha * 0xff;
+		r = argb.red * 0xff;
+		g = argb.green * 0xff;
+		b = argb.blue * 0xff;
 		
 		color.red = (color.red * (0xFF - a) + r * 0x101 * a) / 0xFF;
 		color.green = (color.green * (0xFF - a) + g * 0x101 * a) / 0xFF;
@@ -1552,6 +1565,7 @@ gnome_bg_set_surface_as_root (GdkScreen *screen, cairo_surface_t *surface)
 	int      screen_num;
 
 	g_return_if_fail (screen != NULL);
+	g_return_if_fail (surface != NULL);
 	g_return_if_fail (cairo_surface_get_type (surface) == CAIRO_SURFACE_TYPE_XLIB);
 
 	screen_num = gdk_screen_get_number (screen);
@@ -3012,7 +3026,7 @@ create_thumbnail_for_filename (GnomeDesktopThumbnailFactory *factory,
 			int orig_width = gdk_pixbuf_get_width (orig);
 			int orig_height = gdk_pixbuf_get_height (orig);
 			
-			result = pixbuf_scale_to_fit (orig, 128, 128);
+			result = pixbuf_scale_to_fit (orig, THUMBNAIL_SIZE, THUMBNAIL_SIZE);
 			
 			g_object_set_data_full (G_OBJECT (result), "gnome-thumbnail-height",
 						g_strdup_printf ("%d", orig_height), g_free);
